@@ -22,7 +22,6 @@ const St = imports.gi.St;
 const Clutter = imports.gi.Clutter;
 
 const Gettext = imports.gettext.domain("Shortcuts");
-const Mainloop = imports.mainloop;
 
 const ExtensionUtils = imports.misc.extensionUtils;
 
@@ -31,38 +30,39 @@ const Config = imports.misc.config;
 const Main = imports.ui.main;
 
 const Me = ExtensionUtils.getCurrentExtension();
-const Convenience = Me.imports.convenience;
 const _ = Gettext.gettext;
 
 let button, stage, panel_panel, left_panel, right_panel;
 let _isAdded, _visible, _visibility;
+let _custom_shortcut_handler, _transparency_handler, _visibility_handler;
 
 /**
  * Initialises the plugin.
  */
 function init() {
   ExtensionUtils.initTranslations("Shortcuts");
-  this._settings = Convenience.getSettings();
-  this._settings.connect("changed::show-icon", this._toggleIcon.bind(this));
-  this._settings.connect(
-    "changed::use-custom-shortcuts",
-    this._setShortcutsFile.bind(this)
-  );
-  this._settings.connect(
-    "changed::use-transparency",
-    this._setTransparency.bind(this)
-  );
-  this._settings.connect(
-    "changed::visibility",
-    this._setTransparency.bind(this)
-  );
-  _isAdded = false;
 }
 
 /*
  * Enables the plugin by adding listeners and icons as necessary
  */
 function enable() {
+  this._settings = ExtensionUtils.getSettings("org.gnome.shell.extensions.shortcuts");
+  this._settings.connect("changed::show-icon", this._toggleIcon.bind(this));
+  _custom_shortcut_handler = this._settings.connect(
+    "changed::use-custom-shortcuts",
+    this._setShortcutsFile.bind(this)
+  );
+  _transparency_handler = this._settings.connect(
+    "changed::use-transparency",
+    this._setTransparency.bind(this)
+  );
+  _visibility_handler = this._settings.connect(
+    "changed::visibility",
+    this._setTransparency.bind(this)
+  );
+  _isAdded = false;
+
   Main.overview._specialToggle = function (evt) {
     _toggleShortcuts();
   };
@@ -78,6 +78,12 @@ function enable() {
  * Removes all traces of the listeners and icons that the extension created
  */
 function disable() {
+  this._settings.disconnect(_custom_shortcut_handler);
+  this._settings.disconnect(_transparency_handler);
+  this._settings.disconnect(_visibility_handler);
+  if (this._settings) {
+    delete this._settings;
+  }
   if (_isAdded) {
     Main.panel._rightBox.remove_child(button);
     _isAdded = false;
