@@ -15,28 +15,19 @@
  * along with Shortcuts.  If not, see <http://www.gnu.org/licenses/>.
  **********************************************************************/
 
-const GLib = imports.gi.GLib;
-const GObject = imports.gi.GObject;
-const Gio = imports.gi.Gio;
-const Gtk = imports.gi.Gtk;
-
-const Lang = imports.lang;
+const { Gtk, Gio, GObject } = imports.gi;
 const Gettext = imports.gettext.domain("Shortcuts");
 const _ = Gettext.gettext;
-
 const ExtensionUtils = imports.misc.extensionUtils;
 const Me = ExtensionUtils.getCurrentExtension();
-
 const Config = imports.misc.config;
-
-let Settings;
+const g_schema = "org.gnome.shell.extensions.shortcuts";
 
 /**
  * Initialises the preferences widget
  */
 function init() {
   ExtensionUtils.initTranslations("Shortcuts");
-  Settings = ExtensionUtils.getSettings("org.gnome.shell.extensions.shortcuts");
 }
 
 /**
@@ -54,19 +45,28 @@ function buildPrefsWidget() {
   }
 }
 
+/**
+ * Builds the preferences widget gnome 42 and newer
+ */
+function fillPreferencesWindow(window) {
+  let widget = new AdwShortcutsPrefsWidget(window);
+
+  return widget.fillPreferencesWindow();
+}
+
 /**
  * Describes the widget that is shown in the extension settings section of
  * GNOME tweek.
  */
 const ShortcutsPrefsWidget3 = new GObject.Class({
-  Name: "Shortcuts.Prefs.Widget",
+  Name: "Shortcuts.Prefs.Widget3",
   GTypeName: "ShortcutsPrefsWidget3",
   Extends: Gtk.Grid,
-
   /**
    * Initalises the widget
    */
   _init: function (params) {
+    this._settings = ExtensionUtils.getSettings(g_schema);
     this.parent(params);
     this.margin = 12;
     this.row_spacing = this.column_spacing = 6;
@@ -76,7 +76,7 @@ const ShortcutsPrefsWidget3 = new GObject.Class({
       label: _("Custom Shortcuts File"),
     });
     this.attach(this.customShortcutsFileCheckButton, 0, 0, 2, 1);
-    Settings.bind(
+    this._settings.bind(
       "use-custom-shortcuts",
       this.customShortcutsFileCheckButton,
       "active",
@@ -87,25 +87,22 @@ const ShortcutsPrefsWidget3 = new GObject.Class({
       action: Gtk.FileChooserAction.OPEN,
     });
     this.shortcutsFile.select_uri(
-      "file://" + Settings.get_string("shortcuts-file")
+      "file://" + this._settings.get_string("shortcuts-file")
     );
     let shortcutsFileFilter = new Gtk.FileFilter();
     this.shortcutsFile.set_filter(shortcutsFileFilter);
     shortcutsFileFilter.add_mime_type("application/json");
-    this.shortcutsFile.connect(
-      "selection_changed",
-      Lang.bind(this, function (sw, data) {
-        let path = String(this.shortcutsFile.get_uri()).slice(7);
-        Settings.set_string("shortcuts-file", path);
-      })
-    );
+    this.shortcutsFile.connect("selection_changed", function (sw, data) {
+      let path = String(this.shortcutsFile.get_uri()).slice(7);
+      this._settings.set_string("shortcuts-file", path);
+    });
     this.attach(this.shortcutsFile, 3, 0, 1, 1);
 
     let showIconCheckButton = new Gtk.CheckButton({
       label: _("Show tray icon"),
       margin_top: 6,
     });
-    Settings.bind(
+    this._settings.bind(
       "show-icon",
       showIconCheckButton,
       "active",
@@ -133,42 +130,42 @@ const ShortcutsPrefsWidget4 = GObject.registerClass(
     _init(params = {}) {
       super._init(params);
 
+      this._settings = ExtensionUtils.getSettings(g_schema);
+
       this._file_chooser_chkbutcusfile.set_active(
-        Settings.get_boolean("use-custom-shortcuts")
+        this._settings.get_boolean("use-custom-shortcuts")
       );
 
-      if (Settings.get_boolean("use-custom-shortcuts") == true) {
+      if (this._settings.get_boolean("use-custom-shortcuts")) {
         this._file_chooser_button.label =
-          "file://" + Settings.get_string("shortcuts-file");
+          "file://" + this._settings.get_string("shortcuts-file");
       }
 
       this._file_chooser_chkbuttrayicon.set_active(
-        Settings.get_boolean("show-icon")
+        this._settings.get_boolean("show-icon")
       );
 
       this._file_chooser_chkbuttransp.set_active(
-        Settings.get_boolean("use-transparency")
+        this._settings.get_boolean("use-transparency")
       );
 
       this.append(
-        buildRange(
+        _buildRange(
           "visibility",
           [0, 100, 1, 70],
           _("Visibility"),
-          _("Visibility in percent")
+          _("Visibility in percent"),
+          this._settings
         )
       );
     }
 
     _onChkCusFileClicked(chkbox) {
-      let parent = chkbox.get_root();
-
-      Settings.set_boolean("use-custom-shortcuts", chkbox.get_active());
+      this._settings.set_boolean("use-custom-shortcuts", chkbox.get_active());
     }
 
     _onChkTrayIconClicked(chkbox) {
-      let parent = chkbox.get_root();
-      Settings.set_boolean("show-icon", chkbox.get_active());
+      this._settings.set_boolean("show-icon", chkbox.get_active());
     }
 
     _onBtnClicked(btn) {
@@ -185,8 +182,7 @@ const ShortcutsPrefsWidget4 = GObject.registerClass(
     }
 
     _onChkTransparencyClicked(chkbox) {
-      let parent = chkbox.get_root();
-      Settings.set_boolean("use-transparency", chkbox.get_active());
+      this._settings.set_boolean("use-transparency", chkbox.get_active());
     }
 
     _onFileChooserResponse(native, response) {
@@ -196,13 +192,148 @@ const ShortcutsPrefsWidget4 = GObject.registerClass(
       let fileURI = native.get_file().get_uri();
       this._file_chooser_button.set_label(fileURI);
       let fileURI2 = fileURI.replace("file://", "");
-      Settings.set_string("shortcuts-file", fileURI2);
+      this._settings.set_string("shortcuts-file", fileURI2);
     }
   }
 );
 
-function buildRange(key, values, labeltext, tooltip) {
+class AdwShortcutsPrefsWidget {
+  constructor(window) {
+    this._window = window;
+    this._page1 = null;
+    this._rowfilename = null;
+    this._filechooser = null;
+    this._settings = ExtensionUtils.getSettings(g_schema);
+  }
+
+  _onBtnClicked(btn) {
+    let parent = btn.get_root();
+    this._filechooser.set_transient_for(parent);
+
+    let shortcutsFileFilter = new Gtk.FileFilter();
+    this._filechooser.set_filter(shortcutsFileFilter);
+    shortcutsFileFilter.add_mime_type("application/json");
+
+    this._filechooser.title = _("Select shortcut file");
+
+    this._filechooser.show();
+  }
+
+  _onFileChooserResponse(native, response) {
+    if (response !== Gtk.ResponseType.ACCEPT) {
+      return;
+    }
+    let fileURI = native.get_file().get_uri();
+    this._rowfilename.set_title(fileURI);
+    let fileURI2 = fileURI.replace("file://", "");
+    this._settings.set_string("shortcuts-file", fileURI2);
+  }
+
+  fillPreferencesWindow() {
+    const { Adw } = imports.gi;
+    let adwrow;
+    this._page1 = Adw.PreferencesPage.new(); //page
+    this._page1.set_title(_("Shortcuts"));
+    this._page1.set_name("shortcuts_page");
+    this._page1.set_icon_name("folder-symbolic");
+
+    const group1 = Adw.PreferencesGroup.new(); //group
+    group1.set_title(_("Settings"));
+    group1.set_name("shortcuts_group");
+    this._page1.add(group1);
+
+    adwrow = new Adw.ActionRow({ title: _("Custom Shortcuts File") }); //row1
+    group1.add(adwrow);
+    const togglecustomfile = new Gtk.Switch({
+      active: this._settings.get_boolean("use-custom-shortcuts"),
+      valign: Gtk.Align.CENTER,
+    });
+    this._settings.bind(
+      "use-custom-shortcuts",
+      togglecustomfile,
+      "active",
+      Gio.SettingsBindFlags.DEFAULT
+    );
+    adwrow.add_suffix(togglecustomfile);
+    adwrow.activatable_widget = togglecustomfile;
+
+    adwrow = new Adw.ActionRow({
+      title: "file://" + this._settings.get_string("shortcuts-file"),
+    }); //row2
+    group1.add(adwrow);
+    this._rowfilename = adwrow;
+
+    adwrow = new Adw.ActionRow({ title: _("Select shortcut file") }); //row3
+    let buttonfilechooser = new Gtk.Button({
+      label: _("..."),
+      valign: Gtk.Align.CENTER,
+    });
+    buttonfilechooser.connect(
+      "clicked",
+      this._onBtnClicked.bind(this, buttonfilechooser)
+    );
+    this._filechooser = new Gtk.FileChooserNative({
+      title: _("Select shortcut file"),
+      modal: true,
+      action: Gtk.FileChooserAction.OPEN,
+    });
+    this._filechooser.connect(
+      "response",
+      this._onFileChooserResponse.bind(this)
+    );
+    adwrow.add_suffix(buttonfilechooser);
+    adwrow.activatable_widget = buttonfilechooser;
+    group1.add(adwrow);
+
+    adwrow = new Adw.ActionRow({ title: _("Use transparency") }); //row4
+    group1.add(adwrow);
+    const toggleusetransparency = new Gtk.Switch({
+      active: this._settings.get_boolean("use-transparency"),
+      valign: Gtk.Align.CENTER,
+    });
+    this._settings.bind(
+      "use-transparency",
+      toggleusetransparency,
+      "active",
+      Gio.SettingsBindFlags.DEFAULT
+    );
+    adwrow.add_suffix(toggleusetransparency);
+
+    adwrow = new Adw.ActionRow({ title: _("Visibility") }); //row5
+    let rangeslider = _rangeslider(
+      "visibility",
+      [0, 100, 1, 70],
+      _("Visibility in percent"),
+      this._settings
+    );
+    adwrow.add_suffix(rangeslider);
+    adwrow.activatable_widget = rangeslider;
+    group1.add(adwrow);
+    this._window.add(this._page1);
+  }
+}
+
+function _rangeslider(key, values, tooltip, settings) {
   let [min, max, step, defv] = values;
+  let range = Gtk.Scale.new_with_range(
+    Gtk.Orientation.HORIZONTAL,
+    min,
+    max,
+    step
+  );
+  range.tooltip = tooltip;
+  range.set_value(settings.get_int(key));
+  range.set_draw_value(true);
+  range.add_mark(defv, Gtk.PositionType.BOTTOM, null);
+  range.set_size_request(200, -1);
+
+  range.connect("value-changed", function (slider) {
+    settings.set_int(key, slider.get_value());
+  });
+  return range;
+}
+
+function _buildRange(key, values, labeltext, tooltip, settings) {
   let hbox = new Gtk.Box({
     orientation: Gtk.Orientation.HORIZONTAL,
     spacing: 10,
@@ -210,22 +341,7 @@ function buildRange(key, values, labeltext, tooltip) {
 
   let label = new Gtk.Label({ label: labeltext, xalign: 0 });
 
-  let range = Gtk.Scale.new_with_range(
-    Gtk.Orientation.HORIZONTAL,
-    min,
-    max,
-    step
-  );
-
-  range.tooltip = tooltip;
-  range.set_value(Settings.get_int(key));
-  range.set_draw_value(true);
-  range.add_mark(defv, Gtk.PositionType.BOTTOM, null);
-  range.set_size_request(200, -1);
-
-  range.connect("value-changed", function (slider) {
-    Settings.set_int(key, slider.get_value());
-  });
+  let range = _rangeslider(key, values, tooltip, settings);
 
   hbox.append(label);
   hbox.append(range);
